@@ -35,6 +35,7 @@
 #include "database/Track.hpp"
 #include "database/TrackArtistLink.hpp"
 #include "database/TrackFeatures.hpp"
+#include "database/TrackLyrics.hpp"
 #include "metadata/Exception.hpp"
 #include "metadata/IParser.hpp"
 
@@ -122,6 +123,22 @@ namespace lms::scanner
             return directory;
         }
 
+        db::TrackLyrics::pointer createLyrics(Session& session, const metadata::Lyrics& lyricsInfo)
+        {
+            db::TrackLyrics::pointer lyrics{ session.create<db::TrackLyrics>() };
+
+            lyrics.modify()->setLanguage(!lyricsInfo.language.empty() ? lyricsInfo.language : "xxx");
+            lyrics.modify()->setOffset(lyricsInfo.offset);
+            lyrics.modify()->setDisplayArtist(lyricsInfo.displayArtist);
+            lyrics.modify()->setDisplayTitle(lyricsInfo.displayTitle);
+            if (!lyricsInfo.synchronizedLines.empty())
+                lyrics.modify()->setSynchronizedLines(lyricsInfo.synchronizedLines);
+            else
+                lyrics.modify()->setUnsynchronizedLines(lyricsInfo.unsynchronizedLines);
+
+            return lyrics;
+        }
+
         Artist::pointer createArtist(Session& session, const metadata::Artist& artistInfo)
         {
             Artist::pointer artist{ session.create<Artist>(artistInfo.name) };
@@ -252,7 +269,17 @@ namespace lms::scanner
             }
         }
 
-        Release::pointer getOrCreateRelease(Session& session, std::optional<std::size_t> mediumPosition, const metadata::Release& releaseInfo, const Directory::pointer& currentDirectory)
+        // Compare release level info
+        bool isReleaseMatching(const Release::pointer& candidateRelease, const metadata::Release& releaseInfo)
+        {
+            // TODO: add more criterias?
+            return candidateRelease->getName() == releaseInfo.name
+                && candidateRelease->getSortName() == releaseInfo.sortName
+                && candidateRelease->getTotalDisc() == releaseInfo.mediumCount
+                && candidateRelease->isCompilation() == releaseInfo.isCompilation;
+        }
+
+        Release::pointer getOrCreateRelease(Session& session, const metadata::Release& releaseInfo, const Directory::pointer& currentDirectory)
         {
             Release::pointer release;
 
@@ -265,12 +292,36 @@ namespace lms::scanner
             }
             else if (releaseInfo.name.empty())
             {
-                // No release name (only mbid) -> nothing to de
+                // No release name (only mbid) -> nothing to do
                 return release;
             }
 
             // Fall back on release name (collisions may occur)
-            // First try in the current directory
+            // First try using all sibling directories (case for Album/DiscX), only if the disc number is set
+            const DirectoryId parentDirectoryId{ currentDirectory->getParentDirectoryId() };
+            if (!release && releaseInfo.mediumCount && *releaseInfo.mediumCount > 1 && parentDirectoryId.isValid())
+            {
+                Release::FindParameters params;
+                params.setParentDirectory(parentDirectoryId);
+                params.setName(releaseInfo.name);
+                Release::find(session, params, [&](const Release::pointer& candidateRelease) {
+                    // Already found a candidate
+                    if (release)
+                        return;
+
+                    // Do not fallback on properly tagged releases
+                    if (candidateRelease->getMBID().has_value())
+                        return;
+
+                    if (!isReleaseMatching(candidateRelease, releaseInfo))
+                        return;
+
+                    release = candidateRelease;
+                });
+            }
+
+            // Lastly try in the current directory: we do this at last to have
+            // opportunities to merge releases in case of migration / rescan
             if (!release)
             {
                 Release::FindParameters params;
@@ -285,30 +336,7 @@ namespace lms::scanner
                     if (candidateRelease->getMBID().has_value())
                         return;
 
-                    // TODO: add more criterias?
-                    release = candidateRelease;
-                });
-            }
-
-            // second try in another sibling directory (case for Album/DiscX)
-            const DirectoryId parentDirectoryId{ currentDirectory->getParentDirectoryId() };
-            if (!release && mediumPosition && parentDirectoryId.isValid())
-            {
-                Release::FindParameters params;
-                params.setParentDirectory(parentDirectoryId);
-                params.setName(releaseInfo.name);
-                Release::find(session, params, [&](const Release::pointer& candidateRelease) {
-                    // Already found a candidate
-                    if (release)
-                        return;
-
-                    // Do not fallback on properly tagged releases
-                    if (candidateRelease->getMBID().has_value())
-                        return;
-
-                    // Fallback only if the disc number of the current track is not the same
-                    const std::vector<DiscInfo> discs{ candidateRelease->getDiscs() };
-                    if (discs.empty() || std::any_of(discs.begin(), discs.end(), [&](const DiscInfo& discInfo) { return discInfo.position == *mediumPosition; }))
+                    if (!isReleaseMatching(candidateRelease, releaseInfo))
                         return;
 
                     release = candidateRelease;
@@ -437,6 +465,12 @@ namespace lms::scanner
                         if (checkImageFileNeedScan(context, path))
                             _fileScanQueue.pushScanRequest(path, FileScanQueue::ScanRequestType::ImageFile);
                     }
+                    else if (core::pathUtils::hasFileAnyExtension(path, _settings.supportedLyricsFileExtensions))
+                    {
+                        fileMatched = true;
+                        if (checkLyricsFileNeedScan(context, path))
+                            _fileScanQueue.pushScanRequest(path, FileScanQueue::ScanRequestType::LyricsFile);
+                    }
 
                     if (fileMatched)
                     {
@@ -544,6 +578,34 @@ namespace lms::scanner
         return true; // need to scan
     }
 
+    bool ScanStepScanFiles::checkLyricsFileNeedScan(ScanContext& context, const std::filesystem::path& file)
+    {
+        ScanStats& stats{ context.stats };
+
+        const Wt::WDateTime lastWriteTime{ retrieveFileGetLastWrite(file) };
+        // Should rarely fail as we are currently iterating it
+        if (!lastWriteTime.isValid())
+        {
+            stats.skips++;
+            return false;
+        }
+
+        if (!context.scanOptions.fullScan)
+        {
+            db::Session& dbSession{ _db.getTLSSession() };
+            auto transaction{ _db.getTLSSession().createReadTransaction() };
+
+            const db::TrackLyrics::pointer lyrics{ db::TrackLyrics::find(dbSession, file) };
+            if (lyrics && lyrics->getLastWriteTime() == lastWriteTime)
+            {
+                stats.skips++;
+                return false;
+            }
+        }
+
+        return true; // need to scan
+    }
+
     void ScanStepScanFiles::processFileScanResults(ScanContext& context, std::span<const FileScanResult> scanResults, const ScannerSettings::MediaLibraryInfo& libraryInfo)
     {
         LMS_SCOPED_TRACE_OVERVIEW("Scanner", "ProcessScanResults");
@@ -566,6 +628,11 @@ namespace lms::scanner
                 context.stats.scans++;
                 processImageFileScanData(context, scanResult.path, scanData->has_value() ? &scanData->value() : nullptr, libraryInfo);
             }
+            else if (const LyricsFileScanData * scanData{ std::get_if<LyricsFileScanData>(&scanResult.scanData) })
+            {
+                context.stats.scans++;
+                processLyricsFileScanData(context, scanResult.path, scanData->has_value() ? &scanData->value() : nullptr, libraryInfo);
+            }
         }
     }
 
@@ -737,7 +804,7 @@ namespace lms::scanner
 
         track.modify()->setScanVersion(_settings.scanVersion);
         if (trackMetadata->medium && trackMetadata->medium->release)
-            track.modify()->setRelease(getOrCreateRelease(dbSession, trackMetadata->medium->position, *trackMetadata->medium->release, directory));
+            track.modify()->setRelease(getOrCreateRelease(dbSession, *trackMetadata->medium->release, directory));
         else
             track.modify()->setRelease({});
         track.modify()->setTotalTrack(trackMetadata->medium ? trackMetadata->medium->trackCount : std::nullopt);
@@ -772,6 +839,13 @@ namespace lms::scanner
         track.modify()->setTrackReplayGain(trackMetadata->replayGain);
         track.modify()->setArtistDisplayName(trackMetadata->artistDisplayName);
 
+        track.modify()->clearEmbeddedLyrics();
+        for (const metadata::Lyrics& lyricsInfo : trackMetadata->lyrics)
+        {
+            db::TrackLyrics::pointer lyrics{ createLyrics(dbSession, lyricsInfo) };
+            track.modify()->addLyrics(lyrics);
+        }
+
         if (added)
         {
             LMS_LOG(DBUPDATER, DEBUG, "Added audio file '" << file.string() << "'");
@@ -840,4 +914,70 @@ namespace lms::scanner
             stats.updates++;
         }
     }
+
+    void ScanStepScanFiles::processLyricsFileScanData(ScanContext& context, const std::filesystem::path& file, const metadata::Lyrics* lyricsInfo, const ScannerSettings::MediaLibraryInfo& libraryInfo)
+    {
+        LMS_SCOPED_TRACE_DETAILED("Scanner", "ProcessImageScanData");
+
+        ScanStats& stats{ context.stats };
+
+        const std::optional<FileInfo> fileInfo{ retrieveFileInfo(file, libraryInfo.rootDirectory) };
+        if (!fileInfo)
+        {
+            stats.skips++;
+            return;
+        }
+
+        db::Session& dbSession{ _db.getTLSSession() };
+        db::TrackLyrics::pointer trackLyrics{ db::TrackLyrics::find(dbSession, file) };
+
+        if (!lyricsInfo)
+        {
+            if (trackLyrics)
+            {
+                trackLyrics.remove();
+                stats.deletions++;
+            }
+            context.stats.errors.emplace_back(file, ScanErrorType::CannotReadLyricsFile);
+            return;
+        }
+
+        bool added;
+        if (!trackLyrics)
+        {
+            trackLyrics = dbSession.create<db::TrackLyrics>();
+            trackLyrics.modify()->setAbsoluteFilePath(file);
+            added = true;
+        }
+        else
+        {
+            added = false;
+        }
+
+        trackLyrics.modify()->setLastWriteTime(fileInfo->lastWriteTime);
+        trackLyrics.modify()->setFileSize(fileInfo->fileSize);
+        trackLyrics.modify()->setLanguage(!lyricsInfo->language.empty() ? lyricsInfo->language : "xxx");
+        trackLyrics.modify()->setOffset(lyricsInfo->offset);
+        trackLyrics.modify()->setDisplayTitle(lyricsInfo->displayTitle);
+        trackLyrics.modify()->setDisplayArtist(lyricsInfo->displayArtist);
+        if (!lyricsInfo->synchronizedLines.empty())
+            trackLyrics.modify()->setSynchronizedLines(lyricsInfo->synchronizedLines);
+        else
+            trackLyrics.modify()->setUnsynchronizedLines(lyricsInfo->unsynchronizedLines);
+
+        MediaLibrary::pointer mediaLibrary{ MediaLibrary::find(dbSession, libraryInfo.id) }; // may be null if settings are updated in // => next scan will correct this
+        trackLyrics.modify()->setDirectory(getOrCreateDirectory(dbSession, file.parent_path(), mediaLibrary));
+
+        if (added)
+        {
+            LMS_LOG(DBUPDATER, DEBUG, "Added external lyrics '" << file.string() << "'");
+            stats.additions++;
+        }
+        else
+        {
+            LMS_LOG(DBUPDATER, DEBUG, "Updated external lyrics '" << file.string() << "'");
+            stats.updates++;
+        }
+    }
+
 } // namespace lms::scanner
