@@ -30,14 +30,15 @@
 #include "core/ITraceLogger.hpp"
 #include "core/Service.hpp"
 #include "core/String.hpp"
+#include "core/SystemPaths.hpp"
 #include "core/WtLogger.hpp"
 #include "database/Db.hpp"
 #include "database/Session.hpp"
 #include "image/Image.hpp"
+#include "services/artwork/IArtworkService.hpp"
 #include "services/auth/IAuthTokenService.hpp"
 #include "services/auth/IEnvService.hpp"
 #include "services/auth/IPasswordService.hpp"
-#include "services/cover/ICoverService.hpp"
 #include "services/feedback/IFeedbackService.hpp"
 #include "services/recommendation/IPlaylistGeneratorService.hpp"
 #include "services/recommendation/IRecommendationService.hpp"
@@ -50,180 +51,204 @@
 
 namespace lms
 {
-    std::size_t getThreadCount()
+    namespace
     {
-        const unsigned long configHttpServerThreadCount{ core::Service<core::IConfig>::get()->getULong("http-server-thread-count", 0) };
-
-        // Reserve at least 2 threads since we still have some blocking IO (for example when reading from ffmpeg)
-        return configHttpServerThreadCount ? configHttpServerThreadCount : std::max<unsigned long>(2, std::thread::hardware_concurrency());
-    }
-
-    core::logging::Severity getLogMinSeverity()
-    {
-        std::string_view minSeverity{ core::Service<core::IConfig>::get()->getString("log-min-severity", "info") };
-
-        if (minSeverity == "debug")
-            return core::logging::Severity::DEBUG;
-        else if (minSeverity == "info")
-            return core::logging::Severity::INFO;
-        else if (minSeverity == "warning")
-            return core::logging::Severity::WARNING;
-        else if (minSeverity == "error")
-            return core::logging::Severity::ERROR;
-        else if (minSeverity == "fatal")
-            return core::logging::Severity::FATAL;
-
-        throw core::LmsException{ "Invalid config value for 'log-min-severity'" };
-    }
-
-    std::optional<core::tracing::Level> getTracingLevel()
-    {
-        std::string_view tracingLevel{ core::Service<core::IConfig>::get()->getString("tracing-level", "disabled") };
-
-        if (tracingLevel == "disabled")
-            return std::nullopt;
-        else if (tracingLevel == "overview")
-            return core::tracing::Level::Overview;
-        else if (tracingLevel == "detailed")
-            return core::tracing::Level::Detailed;
-
-        throw core::LmsException{ "Invalid config value for 'tracing-level'" };
-    }
-
-    std::vector<std::string> generateWtConfig(std::string execPath, core::logging::Severity minSeverity)
-    {
-        std::vector<std::string> args;
-
-        const std::filesystem::path wtConfigPath{ core::Service<core::IConfig>::get()->getPath("working-dir") / "wt_config.xml" };
-        const std::filesystem::path wtLogFilePath{ core::Service<core::IConfig>::get()->getPath("log-file", "/var/log/lms.log") };
-        const std::filesystem::path wtAccessLogFilePath{ core::Service<core::IConfig>::get()->getPath("access-log-file", "/var/log/lms.access.log") };
-        const std::filesystem::path wtResourcesPath{ core::Service<core::IConfig>::get()->getPath("wt-resources", "/usr/share/Wt/resources") };
-
-        args.push_back(execPath);
-        args.push_back("--config=" + wtConfigPath.string());
-        args.push_back("--docroot=" + std::string{ core::Service<core::IConfig>::get()->getString("docroot") });
-        args.push_back("--approot=" + std::string{ core::Service<core::IConfig>::get()->getString("approot") });
-        args.push_back("--deploy-path=" + std::string{ core::Service<core::IConfig>::get()->getString("deploy-path", "/") });
-        if (!wtResourcesPath.empty())
-            args.push_back("--resources-dir=" + wtResourcesPath.string());
-
-        if (core::Service<core::IConfig>::get()->getBool("tls-enable", false))
+        std::size_t getThreadCount()
         {
-            args.push_back("--https-port=" + std::to_string(core::Service<core::IConfig>::get()->getULong("listen-port", 5082)));
-            args.push_back("--https-address=" + std::string{ core::Service<core::IConfig>::get()->getString("listen-addr", "0.0.0.0") });
-            args.push_back("--ssl-certificate=" + std::string{ core::Service<core::IConfig>::get()->getString("tls-cert") });
-            args.push_back("--ssl-private-key=" + std::string{ core::Service<core::IConfig>::get()->getString("tls-key") });
-            args.push_back("--ssl-tmp-dh=" + std::string{ core::Service<core::IConfig>::get()->getString("tls-dh") });
-        }
-        else
-        {
-            args.push_back("--http-port=" + std::to_string(core::Service<core::IConfig>::get()->getULong("listen-port", 5082)));
-            args.push_back("--http-address=" + std::string{ core::Service<core::IConfig>::get()->getString("listen-addr", "0.0.0.0") });
+            const unsigned long configHttpServerThreadCount{ core::Service<core::IConfig>::get()->getULong("http-server-thread-count", 0) };
+
+            // Reserve at least 2 threads since we still have some blocking IO (for example when reading from ffmpeg)
+            return configHttpServerThreadCount ? configHttpServerThreadCount : std::max<unsigned long>(2, std::thread::hardware_concurrency());
         }
 
-        if (!wtAccessLogFilePath.empty())
-            args.push_back("--accesslog=" + wtAccessLogFilePath.string());
-
-        args.push_back("--threads=" + std::to_string(getThreadCount()));
-
-        // Generate the wt_config.xml file
-        boost::property_tree::ptree pt;
-
-        pt.put("server.application-settings.<xmlattr>.location", "*");
-        pt.put("server.application-settings.log-file", wtLogFilePath.string());
-
-        // log-config
-        pt.put("server.application-settings.log-config", core::logging::WtLogger::computeLogConfig(minSeverity));
-        pt.put("server.application-settings.behind-reverse-proxy", core::Service<core::IConfig>::get()->getBool("behind-reverse-proxy", false));
-
+        core::logging::Severity getLogMinSeverity()
         {
-            boost::property_tree::ptree viewport;
-            viewport.put("<xmlattr>.name", "viewport");
-            viewport.put("<xmlattr>.content", "width=device-width, initial-scale=1, user-scalable=no");
-            pt.add_child("server.application-settings.head-matter.meta", viewport);
-        }
-        {
-            boost::property_tree::ptree themeColor;
-            themeColor.put("<xmlattr>.name", "theme-color");
-            themeColor.put("<xmlattr>.content", "#303030");
-            pt.add_child("server.application-settings.head-matter.meta", themeColor);
+            std::string_view minSeverity{ core::Service<core::IConfig>::get()->getString("log-min-severity", "info") };
+
+            if (minSeverity == "debug")
+                return core::logging::Severity::DEBUG;
+            else if (minSeverity == "info")
+                return core::logging::Severity::INFO;
+            else if (minSeverity == "warning")
+                return core::logging::Severity::WARNING;
+            else if (minSeverity == "error")
+                return core::logging::Severity::ERROR;
+            else if (minSeverity == "fatal")
+                return core::logging::Severity::FATAL;
+
+            throw core::LmsException{ "Invalid config value for 'log-min-severity'" };
         }
 
+        std::optional<core::tracing::Level> getTracingLevel()
         {
-            std::ofstream oss{ wtConfigPath.string().c_str(), std::ios::out };
-            if (!oss)
-                throw core::LmsException{ "Can't open '" + wtConfigPath.string() + "' for writing!" };
+            std::string_view tracingLevel{ core::Service<core::IConfig>::get()->getString("tracing-level", "disabled") };
 
-            boost::property_tree::xml_parser::write_xml(oss, pt);
+            if (tracingLevel == "disabled")
+                return std::nullopt;
+            else if (tracingLevel == "overview")
+                return core::tracing::Level::Overview;
+            else if (tracingLevel == "detailed")
+                return core::tracing::Level::Detailed;
 
-            if (!oss)
-                throw core::LmsException{ "Can't write in file '" + wtConfigPath.string() + "', no space left?" };
+            throw core::LmsException{ "Invalid config value for 'tracing-level'" };
         }
 
-        return args;
-    }
+        std::vector<std::string> generateWtConfig(std::string execPath, core::logging::Severity minSeverity)
+        {
+            std::vector<std::string> args;
 
-    void proxyScannerEventsToApplication(scanner::IScannerService& scanner, Wt::WServer& server)
-    {
-        auto postAll{ [](Wt::WServer& server, std::function<void()> cb) {
-            server.postAll([cb = std::move(cb)] {
-                // may be nullptr, see https://redmine.webtoolkit.eu/issues/8202
-                if (LmsApp)
-                    cb();
-            });
-        } };
+            const std::filesystem::path wtConfigPath{ core::Service<core::IConfig>::get()->getPath("working-dir") / "wt_config.xml" };
+            const std::filesystem::path wtLogFilePath{ core::Service<core::IConfig>::get()->getPath("log-file", "/var/log/lms.log") };
+            const std::filesystem::path wtAccessLogFilePath{ core::Service<core::IConfig>::get()->getPath("access-log-file", "/var/log/lms.access.log") };
+            const std::filesystem::path wtResourcesPath{ core::Service<core::IConfig>::get()->getPath("wt-resources", "/usr/share/Wt/resources") };
 
-        scanner.getEvents().scanAborted.connect([&] {
-            postAll(server, [] {
-                LmsApp->getScannerEvents().scanAborted.emit();
-                LmsApp->triggerUpdate();
-            });
-        });
+            args.push_back(execPath);
+            args.push_back("--config=" + wtConfigPath.string());
+            args.push_back("--docroot=" + std::string{ core::Service<core::IConfig>::get()->getString("docroot") });
+            args.push_back("--approot=" + std::string{ core::Service<core::IConfig>::get()->getString("approot") });
+            args.push_back("--deploy-path=" + std::string{ core::Service<core::IConfig>::get()->getString("deploy-path", "/") });
+            if (!wtResourcesPath.empty())
+                args.push_back("--resources-dir=" + wtResourcesPath.string());
 
-        scanner.getEvents().scanStarted.connect([&] {
-            postAll(server, [] {
-                LmsApp->getScannerEvents().scanStarted.emit();
-                LmsApp->triggerUpdate();
-            });
-        });
+            if (core::Service<core::IConfig>::get()->getBool("tls-enable", false))
+            {
+                args.push_back("--https-port=" + std::to_string(core::Service<core::IConfig>::get()->getULong("listen-port", 5082)));
+                args.push_back("--https-address=" + std::string{ core::Service<core::IConfig>::get()->getString("listen-addr", "0.0.0.0") });
+                args.push_back("--ssl-certificate=" + std::string{ core::Service<core::IConfig>::get()->getString("tls-cert") });
+                args.push_back("--ssl-private-key=" + std::string{ core::Service<core::IConfig>::get()->getString("tls-key") });
+                args.push_back("--ssl-tmp-dh=" + std::string{ core::Service<core::IConfig>::get()->getString("tls-dh") });
+            }
+            else
+            {
+                args.push_back("--http-port=" + std::to_string(core::Service<core::IConfig>::get()->getULong("listen-port", 5082)));
+                args.push_back("--http-address=" + std::string{ core::Service<core::IConfig>::get()->getString("listen-addr", "0.0.0.0") });
+            }
 
-        scanner.getEvents().scanComplete.connect([&](const scanner::ScanStats& stats) {
-            postAll(server, [=] {
-                LmsApp->getScannerEvents().scanComplete.emit(stats);
-                LmsApp->triggerUpdate();
-            });
-        });
+            if (!wtAccessLogFilePath.empty())
+                args.push_back("--accesslog=" + wtAccessLogFilePath.string());
 
-        scanner.getEvents().scanInProgress.connect([&](const scanner::ScanStepStats& stats) {
-            postAll(server, [=] {
-                LmsApp->getScannerEvents().scanInProgress.emit(stats);
-                LmsApp->triggerUpdate();
-            });
-        });
+            args.push_back("--threads=" + std::to_string(getThreadCount()));
 
-        scanner.getEvents().scanScheduled.connect([&](const Wt::WDateTime dateTime) {
-            postAll(server, [=] {
-                LmsApp->getScannerEvents().scanScheduled.emit(dateTime);
-                LmsApp->triggerUpdate();
+            // Generate the wt_config.xml file
+            boost::property_tree::ptree pt;
+
+            pt.put("server.application-settings.<xmlattr>.location", "*");
+            pt.put("server.application-settings.log-file", wtLogFilePath.string());
+
+            // log-config
+            pt.put("server.application-settings.log-config", core::logging::WtLogger::computeLogConfig(minSeverity));
+
+            // Reverse proxy
+            if (core::Service<core::IConfig>::get()->getBool("behind-reverse-proxy", false))
+            {
+                pt.put("server.application-settings.trusted-proxy-config.original-ip-header", core::Service<core::IConfig>::get()->getString("original-ip-header", "X-Forwarded-For"));
+                core::Service<core::IConfig>::get()->visitStrings("trusted-proxies", [&](std::string_view trustedProxy) {
+                    pt.add("server.application-settings.trusted-proxy-config.trusted-proxies.proxy", std::string{ trustedProxy });
+                },
+                    { "127.0.0.1", "::1" });
+            }
+
+            {
+                boost::property_tree::ptree viewport;
+                viewport.put("<xmlattr>.name", "viewport");
+                viewport.put("<xmlattr>.content", "width=device-width, initial-scale=1, user-scalable=no");
+                pt.add_child("server.application-settings.head-matter.meta", viewport);
+            }
+            {
+                boost::property_tree::ptree themeColor;
+                themeColor.put("<xmlattr>.name", "theme-color");
+                themeColor.put("<xmlattr>.content", "#303030");
+                pt.add_child("server.application-settings.head-matter.meta", themeColor);
+            }
+
+            {
+                std::ofstream oss{ wtConfigPath.string().c_str(), std::ios::out };
+                if (!oss)
+                    throw core::LmsException{ "Can't open '" + wtConfigPath.string() + "' for writing!" };
+
+                boost::property_tree::xml_parser::write_xml(oss, pt);
+
+                if (!oss)
+                    throw core::LmsException{ "Can't write in file '" + wtConfigPath.string() + "', no space left?" };
+            }
+
+            return args;
+        }
+
+        void proxyScannerEventsToApplication(scanner::IScannerService& scanner, Wt::WServer& server)
+        {
+            auto postAll{ [](Wt::WServer& server, std::function<void()> cb) {
+                server.postAll([cb = std::move(cb)] {
+                    // may be nullptr, see https://redmine.webtoolkit.eu/issues/8202
+                    if (LmsApp)
+                        cb();
+                });
+            } };
+
+            scanner.getEvents().scanAborted.connect([&] {
+                postAll(server, [] {
+                    LmsApp->getScannerEvents().scanAborted.emit();
+                    LmsApp->triggerUpdate();
+                });
             });
-        });
-    }
+
+            scanner.getEvents().scanStarted.connect([&] {
+                postAll(server, [] {
+                    LmsApp->getScannerEvents().scanStarted.emit();
+                    LmsApp->triggerUpdate();
+                });
+            });
+
+            scanner.getEvents().scanComplete.connect([&](const scanner::ScanStats& stats) {
+                postAll(server, [=] {
+                    LmsApp->getScannerEvents().scanComplete.emit(stats);
+                    LmsApp->triggerUpdate();
+                });
+            });
+
+            scanner.getEvents().scanInProgress.connect([&](const scanner::ScanStepStats& stats) {
+                postAll(server, [=] {
+                    LmsApp->getScannerEvents().scanInProgress.emit(stats);
+                    LmsApp->triggerUpdate();
+                });
+            });
+
+            scanner.getEvents().scanScheduled.connect([&](const Wt::WDateTime dateTime) {
+                postAll(server, [=] {
+                    LmsApp->getScannerEvents().scanScheduled.emit(dateTime);
+                    LmsApp->triggerUpdate();
+                });
+            });
+        }
+    } // namespace
 
     int main(int argc, char* argv[])
     {
-        std::filesystem::path configFilePath{ "/etc/lms.conf" };
+        std::filesystem::path configFilePath{ core::sysconfDirectory / "lms.conf" };
         int res{ EXIT_FAILURE };
 
         assert(argc > 0);
         assert(argv[0] != NULL);
 
+        auto displayUsage{ [&](std::ostream& os) {
+            os << "Usage:\t" << argv[0] << "\t[conf_file]\n\n"
+               << "Options:\n"
+               << "\tconf_file:\t path to the LMS configuration file (defaults to " << configFilePath << ")\n\n";
+        } };
+
         if (argc == 2)
-            configFilePath = std::string(argv[1], 0, 256);
+        {
+            const std::string_view arg{ argv[1] };
+            if (arg == "-h" || arg == "--help")
+            {
+                displayUsage(std::cout);
+                return EXIT_SUCCESS;
+            }
+            configFilePath = std::string(arg, 0, 256);
+        }
         else if (argc > 2)
         {
-            std::cerr << "Usage:\t" << argv[0] << "\t[conf_file]\n\n"
-                      << "Options:\n"
-                      << "\tconf_file:\t path to the LMS configuration file (defaults to " << configFilePath << ")\n\n";
+            displayUsage(std::cerr);
             return EXIT_FAILURE;
         }
 
@@ -315,7 +340,7 @@ namespace lms
                 throw core::LmsException{ "Bad value '" + authenticationBackend + "' for 'authentication-backend'" };
 
             image::init(argv[0]);
-            core::Service<cover::ICoverService> coverService{ cover::createCoverService(database, server.appRoot() + "/images/unknown-cover.svg") };
+            core::Service<cover::IArtworkService> artworkService{ cover::createArtworkService(database, server.appRoot() + "/images/unknown-cover.svg", server.appRoot() + "/images/unknown-artist.svg") };
             core::Service<recommendation::IRecommendationService> recommendationService{ recommendation::createRecommendationService(database) };
             core::Service<recommendation::IPlaylistGeneratorService> playlistGeneratorService{ recommendation::createPlaylistGeneratorService(database, *recommendationService.get()) };
             core::Service<scanner::IScannerService> scannerService{ scanner::createScannerService(database) };
@@ -323,7 +348,7 @@ namespace lms
             scannerService->getEvents().scanComplete.connect([&] {
                 // Flush cover cache even if no changes:
                 // covers may be external files that changed and we don't keep track of them for now (but we should)
-                coverService->flushCache();
+                artworkService->flushCache();
                 database.getTLSSession().refreshTracingLoggerStats();
             });
 
